@@ -220,13 +220,13 @@ public class CompactionCommand {
       @ShellOption(value = {"--parallelism"}, defaultValue = "3",
           help = "Parallelism for hoodie compaction") final String parallelism,
       @ShellOption(value = "--schemaFilePath",
-          help = "Path for Avro schema file", defaultValue = ShellOption.NULL) final String schemaFilePath,
+          help = "Path for Avro schema file", defaultValue = "") final String schemaFilePath,
       @ShellOption(value = "--sparkMaster", defaultValue = "local",
           help = "Spark Master") String master,
       @ShellOption(value = "--sparkMemory", defaultValue = "4G",
           help = "Spark executor memory") final String sparkMemory,
       @ShellOption(value = "--retry", defaultValue = "1", help = "Number of retries") final String retry,
-      @ShellOption(value = "--compactionInstant", help = "Base path for the target hoodie table",
+      @ShellOption(value = "--compactionInstant", help = "Instant of compaction.request",
           defaultValue = ShellOption.NULL) String compactionInstantTime,
       @ShellOption(value = "--propsFilePath", help = "path to properties file on localfs or dfs with configurations for hoodie client for compacting",
           defaultValue = "") final String propsFilePath,
