@@ -25,6 +25,8 @@ import org.apache.hudi.cli.TableHeader;
 import org.apache.hudi.cli.functional.CLIFunctionalTestHarness;
 import org.apache.hudi.cli.testutils.HoodieTestCommitMetadataGenerator;
 import org.apache.hudi.cli.testutils.HoodieTestReplaceCommitMetadataGenerator;
+import org.apache.hudi.cli.testutils.ShellEvaluationResultUtil;
+import org.apache.hudi.client.HoodieTimelineArchiver;
 import org.apache.hudi.common.config.HoodieMetadataConfig;
 import org.apache.hudi.common.fs.FSUtils;
 import org.apache.hudi.common.model.HoodieTableType;
@@ -41,7 +43,6 @@ import org.apache.hudi.config.HoodieArchivalConfig;
 import org.apache.hudi.config.HoodieCleanConfig;
 import org.apache.hudi.config.HoodieWriteConfig;
 import org.apache.hudi.table.HoodieSparkTable;
-import org.apache.hudi.client.HoodieTimelineArchiver;
 
 import org.apache.hadoop.fs.FileSystem;
 import org.junit.jupiter.api.BeforeEach;
@@ -50,7 +51,9 @@ import org.junit.jupiter.api.Test;
 import org.junit.jupiter.params.ParameterizedTest;
 import org.junit.jupiter.params.provider.EnumSource;
 import org.junit.jupiter.params.provider.ValueSource;
-import org.springframework.shell.core.CommandResult;
+import org.springframework.beans.factory.annotation.Autowired;
+import org.springframework.boot.test.context.SpringBootTest;
+import org.springframework.shell.Shell;
 
 import java.io.IOException;
 import java.util.ArrayList;
@@ -70,8 +73,12 @@ import static org.junit.jupiter.api.Assertions.assertTrue;
  * Test class for {@link org.apache.hudi.cli.commands.CommitsCommand}.
  */
 @Tag("functional")
+@SpringBootTest(properties = {"spring.shell.interactive.enabled=false", "spring.shell.command.script.enabled=false"})
 public class TestCommitsCommand extends CLIFunctionalTestHarness {
 
+  @Autowired
+  private Shell shell;
+
   private String tableName1;
   private String tableName2;
   private String tablePath1;
@@ -170,21 +177,9 @@ public class TestCommitsCommand extends CLIFunctionalTestHarness {
     });
 
     final Map<String, Function<Object, String>> fieldNameToConverterMap = new HashMap<>();
-    fieldNameToConverterMap.put(HoodieTableHeaderFields.HEADER_TOTAL_BYTES_WRITTEN, entry -> {
-      return NumericUtils.humanReadableByteCount((Double.valueOf(entry.toString())));
-    });
+    fieldNameToConverterMap.put(HoodieTableHeaderFields.HEADER_TOTAL_BYTES_WRITTEN, entry -> NumericUtils.humanReadableByteCount((Double.parseDouble(entry.toString()))));
 
-    final TableHeader header = new TableHeader()
-        .addTableHeaderField(HoodieTableHeaderFields.HEADER_COMMIT_TIME)
-        .addTableHeaderField(HoodieTableHeaderFields.HEADER_TOTAL_BYTES_WRITTEN)
-        .addTableHeaderField(HoodieTableHeaderFields.HEADER_TOTAL_FILES_ADDED)
-        .addTableHeaderField(HoodieTableHeaderFields.HEADER_TOTAL_FILES_UPDATED)
-        .addTableHeaderField(HoodieTableHeaderFields.HEADER_TOTAL_PARTITIONS_WRITTEN)
-        .addTableHeaderField(HoodieTableHeaderFields.HEADER_TOTAL_RECORDS_WRITTEN)
-        .addTableHeaderField(HoodieTableHeaderFields.HEADER_TOTAL_UPDATE_RECORDS_WRITTEN)
-        .addTableHeaderField(HoodieTableHeaderFields.HEADER_TOTAL_ERRORS);
-
-    return HoodiePrintHelper.print(header, fieldNameToConverterMap, "", false,
+    return HoodiePrintHelper.print(HoodieTableHeaderFields.getTableHeader(), fieldNameToConverterMap, "", false,
         -1, false, rows);
   }
 
@@ -195,21 +190,76 @@ public class TestCommitsCommand extends CLIFunctionalTestHarness {
   public void testShowCommits() throws Exception {
     Map<String, Integer[]> data = generateData();
 
-    CommandResult cr = shell().executeCommand("commits show");
-    assertTrue(cr.isSuccess());
+    Object result = shell.evaluate(() -> "commits show");
+    assertTrue(ShellEvaluationResultUtil.isSuccess(result));
 
     String expected = generateExpectData(1, data);
     expected = removeNonWordAndStripSpace(expected);
-    String got = removeNonWordAndStripSpace(cr.getResult().toString());
+    String got = removeNonWordAndStripSpace(result.toString());
     assertEquals(expected, got);
   }
 
+  @Test
+  public void testShowCommitsIncludingArchivedTimeline() throws Exception {
+    Map<String, Integer[]> data = generateDataAndArchive(true);
+    data.remove("101");
+    data.remove("102");
+
+    Object result = shell.evaluate(() -> "commits show --includeExtraMetadata true --includeArchivedTimeline true --partition 2015/03/16");
+    assertTrue(ShellEvaluationResultUtil.isSuccess(result));
+
+    String expected = generateExpectDataWithExtraMetadata(1, data);
+    expected = removeNonWordAndStripSpace(expected);
+    String got = removeNonWordAndStripSpace(result.toString());
+    assertEquals(expected, got);
+  }
+
+  private String generateExpectDataWithExtraMetadata(int records, Map<String, Integer[]> data) throws IOException {
+    List<Comparable[]> rows = new ArrayList<>();
+    data.forEach((key, value) -> {
+      for (int i = 0; i < records; i++) {
+        // there are more than 1 partitions, so need to * partitions
+        rows.add(new Comparable[] {HoodieTimeline.COMMIT_ACTION, key, "2015/03/16", HoodieTestCommitMetadataGenerator.DEFAULT_FILEID,
+            HoodieTestCommitMetadataGenerator.DEFAULT_PRE_COMMIT, key.equals("104") ? "20" : "15", "0", "0", key.equals("104") ? "10" : "15",
+            "0", HoodieTestCommitMetadataGenerator.DEFAULT_TOTAL_LOG_BLOCKS, "0", "0", HoodieTestCommitMetadataGenerator.DEFAULT_TOTAL_LOG_RECORDS,
+            "0", HoodieTestCommitMetadataGenerator.DEFAULT_TOTAL_WRITE_BYTES});
+      }
+    });
+
+    final Map<String, Function<Object, String>> fieldNameToConverterMap = new HashMap<>();
+    fieldNameToConverterMap.put(HoodieTableHeaderFields.HEADER_TOTAL_BYTES_WRITTEN, entry -> NumericUtils.humanReadableByteCount((Double.parseDouble(entry.toString()))));
+
+    final TableHeader header = HoodieTableHeaderFields.getTableHeaderWithExtraMetadata();
+
+    return HoodiePrintHelper.print(header, fieldNameToConverterMap, "", false,
+        -1, false, rows);
+  }
+
   /**
    * Test case of 'commits showarchived' command.
    */
   @ParameterizedTest
   @ValueSource(booleans = {true, false})
   public void testShowArchivedCommits(boolean enableMetadataTable) throws Exception {
+    Map<String, Integer[]> data = generateDataAndArchive(enableMetadataTable);
+
+    Object result = shell.evaluate(() -> String.format("commits showarchived --startTs %s --endTs %s", "100", "104"));
+    assertTrue(ShellEvaluationResultUtil.isSuccess(result));
+
+    // archived 101 and 102 instant, generate expect data
+    assertEquals(2, metaClient.reloadActiveTimeline().getCommitsTimeline().countInstants(),
+        "There should 2 instants not be archived!");
+
+    // archived 101 and 102 instants, remove 103 and 104 instant
+    data.remove("103");
+    data.remove("104");
+    String expected = generateExpectData(1, data);
+    expected = removeNonWordAndStripSpace(expected);
+    String got = removeNonWordAndStripSpace(result.toString());
+    assertEquals(expected, got);
+  }
+
+  private Map<String, Integer[]> generateDataAndArchive(boolean enableMetadataTable) throws Exception {
     // Generate archive
     HoodieWriteConfig cfg = HoodieWriteConfig.newBuilder().withPath(tablePath1)
         .withSchema(HoodieTestCommitMetadataGenerator.TRIP_EXAMPLE_SCHEMA).withParallelism(2, 2)
@@ -245,21 +295,7 @@ public class TestCommitsCommand extends CLIFunctionalTestHarness {
     HoodieSparkTable table = HoodieSparkTable.create(cfg, context(), metaClient);
     HoodieTimelineArchiver archiver = new HoodieTimelineArchiver(cfg, table);
     archiver.archiveIfRequired(context());
-
-    CommandResult cr = shell().executeCommand(String.format("commits showarchived --startTs %s --endTs %s", "100", "104"));
-    assertTrue(cr.isSuccess());
-
-    // archived 101 and 102 instant, generate expect data
-    assertEquals(2, metaClient.reloadActiveTimeline().getCommitsTimeline().countInstants(),
-        "There should 2 instants not be archived!");
-
-    // archived 101 and 102 instants, remove 103 and 104 instant
-    data.remove("103");
-    data.remove("104");
-    String expected = generateExpectData(1, data);
-    expected = removeNonWordAndStripSpace(expected);
-    String got = removeNonWordAndStripSpace(cr.getResult().toString());
-    assertEquals(expected, got);
+    return data;
   }
 
   @ParameterizedTest
@@ -302,8 +338,8 @@ public class TestCommitsCommand extends CLIFunctionalTestHarness {
       archiver.archiveIfRequired(context());
     }
 
-    CommandResult cr = shell().executeCommand(String.format("commits showarchived --startTs %s --endTs %s", "160", "174"));
-    assertTrue(cr.isSuccess());
+    Object result = shell.evaluate(() -> String.format("commits showarchived --startTs %s --endTs %s", "160", "174"));
+    assertTrue(ShellEvaluationResultUtil.isSuccess(result));
     assertEquals(3, metaClient.reloadActiveTimeline().getCommitsTimeline().countInstants(),
         "There should 3 instants not be archived!");
 
@@ -313,7 +349,7 @@ public class TestCommitsCommand extends CLIFunctionalTestHarness {
     }
     String expected = generateExpectData(1, data2);
     expected = removeNonWordAndStripSpace(expected);
-    String got = removeNonWordAndStripSpace(cr.getResult().toString());
+    String got = removeNonWordAndStripSpace(result.toString());
     assertEquals(expected, got);
   }
 
@@ -325,8 +361,8 @@ public class TestCommitsCommand extends CLIFunctionalTestHarness {
     Map<String, Integer[]> data = generateData();
 
     String commitInstant = "101";
-    CommandResult cr = shell().executeCommand(String.format("commit showpartitions --commit %s", commitInstant));
-    assertTrue(cr.isSuccess());
+    Object result = shell.evaluate(() -> String.format("commit showpartitions --commit %s", commitInstant));
+    assertTrue(ShellEvaluationResultUtil.isSuccess(result));
 
     Integer[] value = data.get(commitInstant);
     List<Comparable[]> rows = new ArrayList<>();
@@ -351,7 +387,7 @@ public class TestCommitsCommand extends CLIFunctionalTestHarness {
 
     String expected = HoodiePrintHelper.print(header, fieldNameToConverterMap, "", false, -1, false, rows);
     expected = removeNonWordAndStripSpace(expected);
-    String got = removeNonWordAndStripSpace(cr.getResult().toString());
+    String got = removeNonWordAndStripSpace(result.toString());
     assertEquals(expected, got);
   }
 
@@ -360,9 +396,10 @@ public class TestCommitsCommand extends CLIFunctionalTestHarness {
     Map<HoodieInstant, Integer[]> data = generateMixedData();
 
     for (HoodieInstant commitInstant : data.keySet()) {
-      CommandResult cr = shell().executeCommand(String.format("commit showpartitions --commit %s", commitInstant.getTimestamp()));
+      Object result = shell.evaluate(() ->
+              String.format("commit showpartitions --commit %s", commitInstant.getTimestamp()));
 
-      assertTrue(cr.isSuccess());
+      assertTrue(ShellEvaluationResultUtil.isSuccess(result));
 
       Integer[] value = data.get(commitInstant);
       List<Comparable[]> rows = new ArrayList<>();
@@ -387,7 +424,7 @@ public class TestCommitsCommand extends CLIFunctionalTestHarness {
 
       String expected = HoodiePrintHelper.print(header, fieldNameToConverterMap, "", false, -1, false, rows);
       expected = removeNonWordAndStripSpace(expected);
-      String got = removeNonWordAndStripSpace(cr.getResult().toString());
+      String got = removeNonWordAndStripSpace(result.toString());
       assertEquals(expected, got);
     }
   }
@@ -400,8 +437,8 @@ public class TestCommitsCommand extends CLIFunctionalTestHarness {
     Map<String, Integer[]> data = generateData();
 
     String commitInstant = "101";
-    CommandResult cr = shell().executeCommand(String.format("commit showfiles --commit %s", commitInstant));
-    assertTrue(cr.isSuccess());
+    Object result = shell.evaluate(() -> String.format("commit showfiles --commit %s", commitInstant));
+    assertTrue(ShellEvaluationResultUtil.isSuccess(result));
 
     Integer[] value = data.get(commitInstant);
     List<Comparable[]> rows = new ArrayList<>();
@@ -424,7 +461,7 @@ public class TestCommitsCommand extends CLIFunctionalTestHarness {
 
     String expected = HoodiePrintHelper.print(header, new HashMap<>(), "", false, -1, false, rows);
     expected = removeNonWordAndStripSpace(expected);
-    String got = removeNonWordAndStripSpace(cr.getResult().toString());
+    String got = removeNonWordAndStripSpace(result.toString());
     assertEquals(expected, got);
   }
 
@@ -433,8 +470,8 @@ public class TestCommitsCommand extends CLIFunctionalTestHarness {
     Map<HoodieInstant, Integer[]> data = generateMixedData();
 
     for (HoodieInstant commitInstant : data.keySet()) {
-      CommandResult cr = shell().executeCommand(String.format("commit showfiles --commit %s", commitInstant.getTimestamp()));
-      assertTrue(cr.isSuccess());
+      Object result = shell.evaluate(() -> String.format("commit showfiles --commit %s", commitInstant.getTimestamp()));
+      assertTrue(ShellEvaluationResultUtil.isSuccess(result));
 
       Integer[] value = data.get(commitInstant);
       List<Comparable[]> rows = new ArrayList<>();
@@ -457,7 +494,7 @@ public class TestCommitsCommand extends CLIFunctionalTestHarness {
 
       String expected = HoodiePrintHelper.print(header, new HashMap<>(), "", false, -1, false, rows);
       expected = removeNonWordAndStripSpace(expected);
-      String got = removeNonWordAndStripSpace(cr.getResult().toString());
+      String got = removeNonWordAndStripSpace(result.toString());
       assertEquals(expected, got);
     }
   }
@@ -479,15 +516,15 @@ public class TestCommitsCommand extends CLIFunctionalTestHarness {
           Option.of(value[0]), Option.of(value[1]));
     }
 
-    CommandResult cr = shell().executeCommand(String.format("commits compare --path %s", tablePath2));
-    assertTrue(cr.isSuccess());
+    Object result = shell.evaluate(() -> String.format("commits compare --path %s", tablePath2));
+    assertTrue(ShellEvaluationResultUtil.isSuccess(result));
 
     // the latest instant of test_table2 is 101
     List<String> commitsToCatchup = metaClient.getActiveTimeline().findInstantsAfter("101", Integer.MAX_VALUE)
         .getInstants().map(HoodieInstant::getTimestamp).collect(Collectors.toList());
     String expected = String.format("Source %s is ahead by %d commits. Commits to catch up - %s",
         tableName1, commitsToCatchup.size(), commitsToCatchup);
-    assertEquals(expected, cr.getResult().toString());
+    assertEquals(expected, result.toString());
   }
 
   /**
@@ -508,10 +545,10 @@ public class TestCommitsCommand extends CLIFunctionalTestHarness {
           Option.of(value[0]), Option.of(value[1]));
     }
 
-    CommandResult cr = shell().executeCommand(String.format("commits sync --path %s", tablePath2));
-    assertTrue(cr.isSuccess());
+    Object result = shell.evaluate(() -> String.format("commits sync --path %s", tablePath2));
+    assertTrue(ShellEvaluationResultUtil.isSuccess(result));
 
     String expected = String.format("Load sync state between %s and %s", tableName1, tableName2);
-    assertEquals(expected, cr.getResult().toString());
+    assertEquals(expected, result.toString());
   }
 }
