@@ -24,10 +24,10 @@ import org.apache.hudi.common.engine.HoodieEngineContext;
 import org.apache.hudi.common.fs.FSUtils;
 import org.apache.hudi.common.model.HoodieBaseFile;
 import org.apache.hudi.common.model.HoodieRecordPayload;
-import org.apache.hudi.common.model.HoodieTableType;
 import org.apache.hudi.common.table.timeline.HoodieInstant;
 import org.apache.hudi.common.table.timeline.HoodieTimeline;
 import org.apache.hudi.common.table.timeline.TimelineMetadataUtils;
+import org.apache.hudi.common.table.view.FileSystemViewStorageType;
 import org.apache.hudi.common.table.view.TableFileSystemView;
 import org.apache.hudi.common.util.Option;
 import org.apache.hudi.common.util.ValidationUtils;
@@ -65,13 +65,9 @@ public class SavepointActionExecutor<T extends HoodieRecordPayload, I, K, O> ext
 
   @Override
   public HoodieSavepointMetadata execute() {
-    if (table.getMetaClient().getTableType() == HoodieTableType.MERGE_ON_READ) {
-      throw new UnsupportedOperationException("Savepointing is not supported or MergeOnRead table types");
-    }
     Option<HoodieInstant> cleanInstant = table.getCompletedCleanTimeline().lastInstant();
-    HoodieInstant commitInstant = new HoodieInstant(false, HoodieTimeline.COMMIT_ACTION, instantTime);
-    if (!table.getCompletedCommitsTimeline().containsInstant(commitInstant)) {
-      throw new HoodieSavepointException("Could not savepoint non-existing commit " + commitInstant);
+    if (!table.getCompletedCommitsTimeline().containsInstant(instantTime)) {
+      throw new HoodieSavepointException("Could not savepoint non-existing commit " + instantTime);
     }
 
     try {
@@ -89,16 +85,34 @@ public class SavepointActionExecutor<T extends HoodieRecordPayload, I, K, O> ext
       ValidationUtils.checkArgument(HoodieTimeline.compareTimestamps(instantTime, HoodieTimeline.GREATER_THAN_OR_EQUALS, lastCommitRetained),
           "Could not savepoint commit " + instantTime + " as this is beyond the lookup window " + lastCommitRetained);
 
-      context.setJobStatus(this.getClass().getSimpleName(), "Collecting latest files for savepoint " + instantTime);
-      List<String> partitions = FSUtils.getAllPartitionPaths(context, config.getMetadataConfig(), table.getMetaClient().getBasePath());
-      Map<String, List<String>> latestFilesMap = context.mapToPair(partitions, partitionPath -> {
-        // Scan all partitions files with this commit time
-        LOG.info("Collecting latest files in partition path " + partitionPath);
-        TableFileSystemView.BaseFileOnlyView view = table.getBaseFileOnlyView();
-        List<String> latestFiles = view.getLatestBaseFilesBeforeOrOn(partitionPath, instantTime)
-            .map(HoodieBaseFile::getFileName).collect(Collectors.toList());
-        return new ImmutablePair<>(partitionPath, latestFiles);
-      }, null);
+      context.setJobStatus(this.getClass().getSimpleName(), "Collecting latest files for savepoint " + instantTime + " " + table.getConfig().getTableName());
+      TableFileSystemView.BaseFileOnlyView view = table.getBaseFileOnlyView();
+
+      Map<String, List<String>> latestFilesMap;
+      // NOTE: for performance, we have to use different logic here for listing the latest files
+      // before or on the given instant:
+      // (1) using metadata-table-based file listing: instead of parallelizing the partition
+      // listing which incurs unnecessary metadata table reads, we directly read the metadata
+      // table once in a batch manner through the timeline server;
+      // (2) using direct file system listing:  we parallelize the partition listing so that
+      // each partition can be listed on the file system concurrently through Spark.
+      // Note that
+      if (shouldUseBatchLookup(config)) {
+        latestFilesMap = view.getAllLatestBaseFilesBeforeOrOn(instantTime).entrySet().stream()
+            .collect(Collectors.toMap(
+                Map.Entry::getKey,
+                entry -> entry.getValue().map(HoodieBaseFile::getFileName).collect(Collectors.toList())));
+      } else {
+        List<String> partitions = FSUtils.getAllPartitionPaths(context, config.getMetadataConfig(), table.getMetaClient().getBasePath());
+        latestFilesMap = context.mapToPair(partitions, partitionPath -> {
+          // Scan all partitions files with this commit time
+          LOG.info("Collecting latest files in partition path " + partitionPath);
+          List<String> latestFiles = view.getLatestBaseFilesBeforeOrOn(partitionPath, instantTime)
+              .map(HoodieBaseFile::getFileName).collect(Collectors.toList());
+          return new ImmutablePair<>(partitionPath, latestFiles);
+        }, null);
+      }
+
       HoodieSavepointMetadata metadata = TimelineMetadataUtils.convertSavepointMetadata(user, comment, latestFilesMap);
       // Nothing to save in the savepoint
       table.getActiveTimeline().createNewInstant(
@@ -112,4 +126,22 @@ public class SavepointActionExecutor<T extends HoodieRecordPayload, I, K, O> ext
       throw new HoodieSavepointException("Failed to savepoint " + instantTime, e);
     }
   }
+
+  /**
+   * Whether to use batch lookup for listing the latest base files in metadata table.
+   * <p>
+   * Note that metadata table has to be enabled, and the storage type of the file system view
+   * cannot be EMBEDDED_KV_STORE or SPILLABLE_DISK (these two types are not integrated with
+   * metadata table, see HUDI-5612).
+   *
+   * @param config Write configs.
+   * @return {@code true} if using batch lookup; {@code false} otherwise.
+   */
+  private boolean shouldUseBatchLookup(HoodieWriteConfig config) {
+    FileSystemViewStorageType storageType =
+        config.getClientSpecifiedViewStorageConfig().getStorageType();
+    return config.getMetadataConfig().enabled()
+        && !FileSystemViewStorageType.EMBEDDED_KV_STORE.equals(storageType)
+        && !FileSystemViewStorageType.SPILLABLE_DISK.equals(storageType);
+  }
 }
