@@ -24,6 +24,7 @@ import org.apache.hudi.common.model.HoodieCommitMetadata;
 import org.apache.hudi.common.model.HoodieReplaceCommitMetadata;
 import org.apache.hudi.common.model.WriteOperationType;
 import org.apache.hudi.common.table.HoodieTableMetaClient;
+import org.apache.hudi.common.util.CollectionUtils;
 import org.apache.hudi.common.util.Option;
 import org.apache.hudi.exception.HoodieIOException;
 
@@ -39,6 +40,11 @@ import java.util.Set;
 import java.util.stream.Collectors;
 import java.util.stream.Stream;
 
+import static org.apache.hudi.common.table.timeline.HoodieTimeline.COMMIT_ACTION;
+import static org.apache.hudi.common.table.timeline.HoodieTimeline.DELTA_COMMIT_ACTION;
+import static org.apache.hudi.common.table.timeline.HoodieTimeline.REPLACE_COMMIT_ACTION;
+import static org.apache.hudi.common.table.timeline.HoodieTimeline.SAVEPOINT_ACTION;
+
 /**
  * TimelineUtils provides a common way to query incremental meta-data changes for a hoodie table.
  *
@@ -87,15 +93,15 @@ public class TimelineUtils {
   public static List<String> getAffectedPartitions(HoodieTimeline timeline) {
     return timeline.filterCompletedInstants().getInstants().flatMap(s -> {
       switch (s.getAction()) {
-        case HoodieTimeline.COMMIT_ACTION:
-        case HoodieTimeline.DELTA_COMMIT_ACTION:
+        case COMMIT_ACTION:
+        case DELTA_COMMIT_ACTION:
           try {
             HoodieCommitMetadata commitMetadata = HoodieCommitMetadata.fromBytes(timeline.getInstantDetails(s).get(), HoodieCommitMetadata.class);
             return commitMetadata.getPartitionToWriteStats().keySet().stream();
           } catch (IOException e) {
             throw new HoodieIOException("Failed to get partitions written at " + s, e);
-          } 
-        case HoodieTimeline.REPLACE_COMMIT_ACTION:
+          }
+        case REPLACE_COMMIT_ACTION:
           try {
             HoodieReplaceCommitMetadata commitMetadata = HoodieReplaceCommitMetadata.fromBytes(
                 timeline.getInstantDetails(s).get(), HoodieReplaceCommitMetadata.class);
@@ -148,11 +154,11 @@ public class TimelineUtils {
    * Get extra metadata for specified key from latest commit/deltacommit/replacecommit(eg. insert_overwrite) instant.
    */
   public static Option<String> getExtraMetadataFromLatest(HoodieTableMetaClient metaClient, String extraMetadataKey) {
-    return metaClient.getCommitsTimeline().filterCompletedInstants().getReverseOrderedInstants()       
+    return metaClient.getCommitsTimeline().filterCompletedInstants().getReverseOrderedInstants()
         // exclude clustering commits for returning user stored extra metadata 
         .filter(instant -> !isClusteringCommit(metaClient, instant))
         .findFirst().map(instant ->
-        getMetadataValue(metaClient, extraMetadataKey, instant)).orElse(Option.empty());
+            getMetadataValue(metaClient, extraMetadataKey, instant)).orElse(Option.empty());
   }
 
   /**
@@ -170,7 +176,7 @@ public class TimelineUtils {
    */
   public static Map<String, Option<String>> getAllExtraMetadataForKey(HoodieTableMetaClient metaClient, String extraMetadataKey) {
     return metaClient.getCommitsTimeline().filterCompletedInstants().getReverseOrderedInstants().collect(Collectors.toMap(
-          HoodieInstant::getTimestamp, instant -> getMetadataValue(metaClient, extraMetadataKey, instant)));
+        HoodieInstant::getTimestamp, instant -> getMetadataValue(metaClient, extraMetadataKey, instant)));
   }
 
   private static Option<String> getMetadataValue(HoodieTableMetaClient metaClient, String extraMetadataKey, HoodieInstant instant) {
@@ -184,17 +190,17 @@ public class TimelineUtils {
       throw new HoodieIOException("Unable to parse instant metadata " + instant, e);
     }
   }
-  
-  private static boolean isClusteringCommit(HoodieTableMetaClient metaClient, HoodieInstant instant) {
+
+  public static boolean isClusteringCommit(HoodieTableMetaClient metaClient, HoodieInstant instant) {
     try {
-      if (HoodieTimeline.REPLACE_COMMIT_ACTION.equals(instant.getAction())) {
+      if (REPLACE_COMMIT_ACTION.equals(instant.getAction())) {
         // replacecommit is used for multiple operations: insert_overwrite/cluster etc. 
         // Check operation type to see if this instant is related to clustering.
         HoodieReplaceCommitMetadata replaceMetadata = HoodieReplaceCommitMetadata.fromBytes(
             metaClient.getActiveTimeline().getInstantDetails(instant).get(), HoodieReplaceCommitMetadata.class);
         return WriteOperationType.CLUSTER.equals(replaceMetadata.getOperationType());
       }
-      
+
       return false;
     } catch (IOException e) {
       throw new HoodieIOException("Unable to read instant information: " + instant + " for " + metaClient.getBasePath(), e);
@@ -209,4 +215,84 @@ public class TimelineUtils {
     }
     return activeTimeline;
   }
+
+  /**
+   * Returns a Hudi timeline with commits after the given instant time (exclusive).
+   *
+   * @param metaClient                {@link HoodieTableMetaClient} instance.
+   * @param exclusiveStartInstantTime Start instant time (exclusive).
+   * @return Hudi timeline.
+   */
+  public static HoodieTimeline getCommitsTimelineAfter(
+      HoodieTableMetaClient metaClient, String exclusiveStartInstantTime) {
+    HoodieActiveTimeline activeTimeline = metaClient.getActiveTimeline();
+    HoodieDefaultTimeline timeline =
+        activeTimeline.isBeforeTimelineStarts(exclusiveStartInstantTime)
+            ? metaClient.getArchivedTimeline(exclusiveStartInstantTime)
+            .mergeTimeline(activeTimeline)
+            : activeTimeline;
+    return timeline.getCommitsTimeline()
+        .findInstantsAfter(exclusiveStartInstantTime, Integer.MAX_VALUE);
+  }
+  
+  /**
+   * Returns the commit metadata of the given instant.
+   *
+   * @param instant  The hoodie instant
+   * @param timeline The timeline
+   * @return the commit metadata
+   */
+  public static HoodieCommitMetadata getCommitMetadata(
+      HoodieInstant instant,
+      HoodieTimeline timeline) throws IOException {
+    byte[] data = timeline.getInstantDetails(instant).get();
+    if (instant.getAction().equals(REPLACE_COMMIT_ACTION)) {
+      return HoodieReplaceCommitMetadata.fromBytes(data, HoodieReplaceCommitMetadata.class);
+    } else {
+      return HoodieCommitMetadata.fromBytes(data, HoodieCommitMetadata.class);
+    }
+  }
+
+  /**
+   * Gets the qualified earliest instant from the active timeline of the data table
+   * for the archival in metadata table.
+   * <p>
+   * the qualified earliest instant is chosen as the earlier one between the earliest
+   * commit (COMMIT, DELTA_COMMIT, and REPLACE_COMMIT only, considering non-savepoint
+   * commit only if enabling archive beyond savepoint) and the earliest inflight
+   * instant (all actions).
+   *
+   * @param dataTableActiveTimeline      the active timeline of the data table.
+   * @param shouldArchiveBeyondSavepoint whether to archive beyond savepoint.
+   * @return the instant meeting the requirement.
+   */
+  public static Option<HoodieInstant> getEarliestInstantForMetadataArchival(
+      HoodieActiveTimeline dataTableActiveTimeline, boolean shouldArchiveBeyondSavepoint) {
+    // This is for commits only, not including CLEAN, ROLLBACK, etc.
+    // When archive beyond savepoint is enabled, there are chances that there could be holes
+    // in the timeline due to archival and savepoint interplay.  So, the first non-savepoint
+    // commit in the data timeline is considered as beginning of the active timeline.
+    Option<HoodieInstant> earliestCommit = shouldArchiveBeyondSavepoint
+        ? dataTableActiveTimeline.getTimelineOfActions(
+            CollectionUtils.createSet(
+                COMMIT_ACTION, DELTA_COMMIT_ACTION, REPLACE_COMMIT_ACTION, SAVEPOINT_ACTION))
+        .getFirstNonSavepointCommit()
+        : dataTableActiveTimeline.getCommitsTimeline().firstInstant();
+    // This is for all instants which are in-flight
+    Option<HoodieInstant> earliestInflight =
+        dataTableActiveTimeline.filterInflightsAndRequested().firstInstant();
+
+    if (earliestCommit.isPresent() && earliestInflight.isPresent()) {
+      if (earliestCommit.get().compareTo(earliestInflight.get()) < 0) {
+        return earliestCommit;
+      }
+      return earliestInflight;
+    } else if (earliestCommit.isPresent()) {
+      return earliestCommit;
+    } else if (earliestInflight.isPresent()) {
+      return earliestInflight;
+    } else {
+      return Option.empty();
+    }
+  }
 }
