@@ -39,6 +39,7 @@ import org.apache.log4j.Logger;
 
 import java.io.Serializable;
 import java.util.List;
+import java.util.Map;
 import java.util.stream.Stream;
 
 /**
@@ -145,6 +146,12 @@ public class PriorityBasedFileSystemView implements SyncableFileSystemView, Seri
         secondaryView::getLatestBaseFilesBeforeOrOn);
   }
 
+  @Override
+  public Map<String, Stream<HoodieBaseFile>> getAllLatestBaseFilesBeforeOrOn(String maxCommitTime) {
+    return execute(maxCommitTime, preferredView::getAllLatestBaseFilesBeforeOrOn,
+        secondaryView::getAllLatestBaseFilesBeforeOrOn);
+  }
+
   @Override
   public Option<HoodieBaseFile> getLatestBaseFile(String partitionPath, String fileId) {
     return execute(partitionPath, fileId, preferredView::getLatestBaseFile, secondaryView::getLatestBaseFile);
@@ -239,6 +246,7 @@ public class PriorityBasedFileSystemView implements SyncableFileSystemView, Seri
   public void reset() {
     preferredView.reset();
     secondaryView.reset();
+    errorOnPreferredView = false;
   }
 
   @Override
@@ -253,8 +261,9 @@ public class PriorityBasedFileSystemView implements SyncableFileSystemView, Seri
 
   @Override
   public void sync() {
-    preferredView.reset();
-    secondaryView.reset();
+    preferredView.sync();
+    secondaryView.sync();
+    errorOnPreferredView = false;
   }
 
   @Override
