@@ -72,7 +72,7 @@ object HoodieOptionConfig {
   /**
    * The mapping of the sql short name key to the hoodie's config key.
    */
-  private lazy val keyMapping: Map[String, String] = {
+  private lazy val sqlOptionKeyToWriteConfigKey: Map[String, String] = {
     HoodieOptionConfig.getClass.getDeclaredFields
         .filter(f => f.getType == classOf[HoodieSQLOption[_]])
         .map(f => {f.setAccessible(true); f.get(HoodieOptionConfig).asInstanceOf[HoodieSQLOption[_]]})
@@ -80,11 +80,14 @@ object HoodieOptionConfig {
         .toMap
   }
 
+  private lazy val writeConfigKeyToSqlOptionKey: Map[String, String] =
+    sqlOptionKeyToWriteConfigKey.map(f => f._2 -> f._1)
+
   /**
    * The mapping of the sql short name key to the hoodie table config key
    * defined in HoodieTableConfig.
    */
-  private lazy val keyTableConfigMapping: Map[String, String] = {
+  private lazy val sqlOptionKeyToTableConfigKey: Map[String, String] = {
     HoodieOptionConfig.getClass.getDeclaredFields
       .filter(f => f.getType == classOf[HoodieSQLOption[_]])
       .map(f => {f.setAccessible(true); f.get(HoodieOptionConfig).asInstanceOf[HoodieSQLOption[_]]})
@@ -93,41 +96,43 @@ object HoodieOptionConfig {
       .toMap
   }
 
-  private lazy val tableConfigKeyToSqlKey: Map[String, String] =
-    keyTableConfigMapping.map(f => f._2 -> f._1)
+  private lazy val tableConfigKeyToSqlOptionKey: Map[String, String] =
+    sqlOptionKeyToTableConfigKey.map(f => f._2 -> f._1)
 
   /**
    * Mapping of the short sql value to the hoodie's config value
    */
-  private val valueMapping: Map[String, String] = Map (
+  private val sqlOptionValueToWriteConfigValue: Map[String, String] = Map (
     SQL_VALUE_TABLE_TYPE_COW -> DataSourceWriteOptions.COW_TABLE_TYPE_OPT_VAL,
     SQL_VALUE_TABLE_TYPE_MOR -> DataSourceWriteOptions.MOR_TABLE_TYPE_OPT_VAL
   )
 
-  private lazy val reverseValueMapping = valueMapping.map(f => f._2 -> f._1)
+  private lazy val writeConfigValueToSqlOptionValue = sqlOptionValueToWriteConfigValue.map(f => f._2 -> f._1)
 
   def withDefaultSqlOptions(options: Map[String, String]): Map[String, String] = defaultSqlOptions ++ options
 
   /**
-   * Mapping the sql's short name key/value in the options to the hoodie's config key/value.
-   * @param options
-   * @return
+   * Map SQL options to data source write configs.
    */
-  def mappingSqlOptionToHoodieParam(options: Map[String, String]): Map[String, String] = {
+  def mapSqlOptionsToDataSourceWriteConfigs(options: Map[String, String]): Map[String, String] = {
     options.map (kv =>
-      keyMapping.getOrElse(kv._1, kv._1) -> valueMapping.getOrElse(kv._2, kv._2))
+      sqlOptionKeyToWriteConfigKey.getOrElse(kv._1, kv._1) -> sqlOptionValueToWriteConfigValue.getOrElse(kv._2, kv._2))
   }
 
   /**
-   * Mapping the sql options to the hoodie table config which used to store to the hoodie
-   * .properties when create the table.
-   * @param options
-   * @return
+   * Mapping the data source write configs to SQL options.
    */
-  def mappingSqlOptionToTableConfig(options: Map[String, String]): Map[String, String] = {
+  def mapDataSourceWriteOptionsToSqlOptions(options: Map[String, String]): Map[String, String] = {
+    options.map(kv => writeConfigKeyToSqlOptionKey.getOrElse(kv._1, kv._1) -> writeConfigValueToSqlOptionValue.getOrElse(kv._2, kv._2))
+  }
+
+  /**
+   * Map SQL options to table configs.
+   */
+  def mapSqlOptionsToTableConfigs(options: Map[String, String]): Map[String, String] = {
     options.map { case (k, v) =>
-      if (keyTableConfigMapping.contains(k)) {
-        keyTableConfigMapping(k) -> valueMapping.getOrElse(v, v)
+      if (sqlOptionKeyToTableConfigKey.contains(k)) {
+        sqlOptionKeyToTableConfigKey(k) -> sqlOptionValueToWriteConfigValue.getOrElse(v, v)
       } else {
         k -> v
       }
@@ -135,10 +140,10 @@ object HoodieOptionConfig {
   }
 
   /**
-   * Mapping the table config (loaded from the hoodie.properties) to the sql options.
+   * Map table configs to SQL options.
    */
-  def mappingTableConfigToSqlOption(options: Map[String, String]): Map[String, String] = {
-    options.map(kv => tableConfigKeyToSqlKey.getOrElse(kv._1, kv._1) -> reverseValueMapping.getOrElse(kv._2, kv._2))
+  def mapTableConfigsToSqlOptions(options: Map[String, String]): Map[String, String] = {
+    options.map(kv => tableConfigKeyToSqlOptionKey.getOrElse(kv._1, kv._1) -> writeConfigValueToSqlOptionValue.getOrElse(kv._2, kv._2))
   }
 
   val defaultSqlOptions: Map[String, String] = {
@@ -150,42 +155,30 @@ object HoodieOptionConfig {
       .toMap
   }
 
-  /**
-   * Get the primary key from the table options.
-   * @param options
-   * @return
-   */
-  def getPrimaryColumns(options: Map[String, String]): Array[String] = {
-    val params = mappingSqlOptionToHoodieParam(options)
-    params.get(DataSourceWriteOptions.RECORDKEY_FIELD.key)
-      .map(_.split(",").filter(_.nonEmpty))
-      .getOrElse(Array.empty)
-  }
-
   /**
    * Get the table type from the table options.
    * @param options
    * @return
    */
   def getTableType(options: Map[String, String]): String = {
-    val params = mappingSqlOptionToHoodieParam(options)
+    val params = mapSqlOptionsToDataSourceWriteConfigs(options)
     params.getOrElse(DataSourceWriteOptions.TABLE_TYPE.key,
       DataSourceWriteOptions.TABLE_TYPE.defaultValue)
   }
 
   def getPreCombineField(options: Map[String, String]): Option[String] = {
-    val params = mappingSqlOptionToHoodieParam(options)
+    val params = mapSqlOptionsToDataSourceWriteConfigs(options)
     params.get(DataSourceWriteOptions.PRECOMBINE_FIELD.key).filter(_.nonEmpty)
   }
 
   def deleteHoodieOptions(options: Map[String, String]): Map[String, String] = {
-    options.filterNot(_._1.startsWith("hoodie.")).filterNot(kv => keyMapping.contains(kv._1))
+    options.filterNot(_._1.startsWith("hoodie.")).filterNot(kv => sqlOptionKeyToWriteConfigKey.contains(kv._1))
   }
 
   // extract primaryKey, preCombineField, type options
   def extractSqlOptions(options: Map[String, String]): Map[String, String] = {
-    val sqlOptions = mappingTableConfigToSqlOption(options)
-    val targetOptions = keyMapping.keySet -- Set(SQL_PAYLOAD_CLASS.sqlKeyName)
+    val sqlOptions = mapTableConfigsToSqlOptions(options)
+    val targetOptions = sqlOptionKeyToWriteConfigKey.keySet -- Set(SQL_PAYLOAD_CLASS.sqlKeyName)
     sqlOptions.filterKeys(targetOptions.contains)
   }
 
